@@ -1,11 +1,19 @@
 # type: ignore
-# This module is not used for now, so we just skip type check for the sake of simplicity.
+# This module is not used for now, so we just skip type checking for the sake
+# of simplicity.
 """ This file contains parsing routines and object classes to help derive meaning from
 raw lists of tokens from pyparsing. """
 
 import abc
 import logging
+from typing import Any
+from typing import Callable
+from typing import Iterator
 from typing import List
+from typing import Optional
+from typing import Sequence
+from typing import Tuple
+from typing import Type
 
 from certbot import errors
 
@@ -23,24 +31,24 @@ class Parsable:
 
     __metaclass__ = abc.ABCMeta
 
-    def __init__(self, parent=None):
-        self._data: List[object] = []
+    def __init__(self, parent: Optional["Parsable"] = None):
+        self._data: List[Any] = []
         self._tabs = None
         self.parent = parent
 
     @classmethod
-    def parsing_hooks(cls):
-        """Returns object types that this class should be able to `parse` recusrively.
+    def parsing_hooks(cls) -> Tuple[Type["Block"], Type["Sentence"], Type["Statements"]]:
+        """Returns object types that this class should be able to `parse` recursively.
         The order of the objects indicates the order in which the parser should
         try to parse each subitem.
         :returns: A list of Parsable classes.
         :rtype list:
         """
-        return (Block, Sentence, Statements)
+        return Block, Sentence, Statements
 
     @staticmethod
     @abc.abstractmethod
-    def should_parse(lists):
+    def should_parse(lists: Any) -> bool:
         """ Returns whether the contents of `lists` can be parsed into this object.
 
         :returns: Whether `lists` can be parsed as this object.
@@ -49,7 +57,7 @@ class Parsable:
         raise NotImplementedError()
 
     @abc.abstractmethod
-    def parse(self, raw_list, add_spaces=False):
+    def parse(self, raw_list: List[Any], add_spaces: bool = False) -> None:
         """ Loads information into this object from underlying raw_list structure.
         Each Parsable object might make different assumptions about the structure of
         raw_list.
@@ -64,7 +72,8 @@ class Parsable:
         raise NotImplementedError()
 
     @abc.abstractmethod
-    def iterate(self, expanded=False, match=None):
+    def iterate(self, expanded: bool = False,
+                match: Optional[Callable[["Parsable"], bool]] = None) -> Iterator[Any]:
         """ Iterates across this object. If this object is a leaf object, only yields
         itself. If it contains references other parsing objects, and `expanded` is set,
         this function should first yield itself, then recursively iterate across all of them.
@@ -77,7 +86,7 @@ class Parsable:
         raise NotImplementedError()
 
     @abc.abstractmethod
-    def get_tabs(self):
+    def get_tabs(self) -> str:
         """ Guess at the tabbing style of this parsed object, based on whitespace.
 
         If this object is a leaf, it deducts the tabbing based on its own contents.
@@ -90,7 +99,7 @@ class Parsable:
         raise NotImplementedError()
 
     @abc.abstractmethod
-    def set_tabs(self, tabs="    "):
+    def set_tabs(self, tabs: str = "    ") -> None:
         """This tries to set and alter the tabbing of the current object to a desired
         whitespace string. Primarily meant for objects that were constructed, so they
         can conform to surrounding whitespace.
@@ -99,7 +108,7 @@ class Parsable:
         """
         raise NotImplementedError()
 
-    def dump(self, include_spaces=False):
+    def dump(self, include_spaces: bool = False) -> List[Any]:
         """ Dumps back to pyparsing-like list tree. The opposite of `parse`.
 
         Note: if this object has not been modified, `dump` with `include_spaces=True`
@@ -121,17 +130,17 @@ class Statements(Parsable):
     an extra `_trailing_whitespace` string to keep track of the whitespace that does not
     precede any more statements.
     """
-    def __init__(self, parent=None):
+    def __init__(self, parent: Optional[Parsable] = None):
         super().__init__(parent)
         self._trailing_whitespace = None
 
     # ======== Begin overridden functions
 
     @staticmethod
-    def should_parse(lists):
+    def should_parse(lists: Any) -> bool:
         return isinstance(lists, list)
 
-    def set_tabs(self, tabs="    "):
+    def set_tabs(self, tabs: str = "    ") -> None:
         """ Sets the tabbing for this set of statements. Does this by calling `set_tabs`
         on each of the child statements.
 
@@ -144,7 +153,7 @@ class Statements(Parsable):
         if self.parent is not None:
             self._trailing_whitespace = "\n" + self.parent.get_tabs()
 
-    def parse(self, raw_list, add_spaces=False):
+    def parse(self, raw_list: List[Any], add_spaces: bool = False) -> None:
         """ Parses a list of statements.
         Expects all elements in `raw_list` to be parseable by `type(self).parsing_hooks`,
         with an optional whitespace string at the last index of `raw_list`.
@@ -157,14 +166,14 @@ class Statements(Parsable):
             raw_list = raw_list[:-1]
         self._data = [parse_raw(elem, self, add_spaces) for elem in raw_list]
 
-    def get_tabs(self):
+    def get_tabs(self) -> str:
         """ Takes a guess at the tabbing of all contained Statements by retrieving the
         tabbing of the first Statement."""
         if self._data:
             return self._data[0].get_tabs()
         return ""
 
-    def dump(self, include_spaces=False):
+    def dump(self, include_spaces: bool = False) -> List[Any]:
         """ Dumps this object by first dumping each statement, then appending its
         trailing whitespace (if `include_spaces` is set) """
         data = super().dump(include_spaces)
@@ -172,16 +181,16 @@ class Statements(Parsable):
             return data + [self._trailing_whitespace]
         return data
 
-    def iterate(self, expanded=False, match=None):
+    def iterate(self, expanded: bool = False,
+                match: Optional[Callable[["Parsable"], bool]] = None) -> Iterator[Any]:
         """ Combines each statement's iterator.  """
         for elem in self._data:
-            for sub_elem in elem.iterate(expanded, match):
-                yield sub_elem
+            yield from elem.iterate(expanded, match)
 
     # ======== End overridden functions
 
 
-def _space_list(list_):
+def _space_list(list_: Sequence[Any]) -> List[str]:
     """ Inserts whitespace between adjacent non-whitespace tokens. """
     spaced_statement: List[str] = []
     for i in reversed(range(len(list_))):
@@ -197,7 +206,7 @@ class Sentence(Parsable):
     # ======== Begin overridden functions
 
     @staticmethod
-    def should_parse(lists):
+    def should_parse(lists: Any) -> bool:
         """ Returns True if `lists` can be parseable as a `Sentence`-- that is,
         every element is a string type.
 
@@ -205,38 +214,39 @@ class Sentence(Parsable):
 
         :returns: whether this lists is parseable by `Sentence`.
         """
-        return isinstance(lists, list) and len(lists) > 0 and \
-               all(isinstance(elem, str) for elem in lists)
+        return (isinstance(lists, list) and len(lists) > 0 and
+                all(isinstance(elem, str) for elem in lists))
 
-    def parse(self, raw_list, add_spaces=False):
+    def parse(self, raw_list: List[Any], add_spaces: bool = False) -> None:
         """ Parses a list of string types into this object.
         If add_spaces is set, adds whitespace tokens between adjacent non-whitespace tokens."""
         if add_spaces:
             raw_list = _space_list(raw_list)
-        if not isinstance(raw_list, list) or \
-            any(not isinstance(elem, str) for elem in raw_list):
+        if (not isinstance(raw_list, list)
+                or any(not isinstance(elem, str) for elem in raw_list)):
             raise errors.MisconfigurationError("Sentence parsing expects a list of string types.")
         self._data = raw_list
 
-    def iterate(self, expanded=False, match=None):
+    def iterate(self, expanded: bool = False,
+                match: Optional[Callable[[Parsable], bool]] = None) -> Iterator[Any]:
         """ Simply yields itself. """
         if match is None or match(self):
             yield self
 
-    def set_tabs(self, tabs="    "):
+    def set_tabs(self, tabs: str = "    ") -> None:
         """ Sets the tabbing on this sentence. Inserts a newline and `tabs` at the
         beginning of `self._data`. """
         if self._data[0].isspace():
             return
         self._data.insert(0, "\n" + tabs)
 
-    def dump(self, include_spaces=False):
+    def dump(self, include_spaces: bool = False) -> List[Any]:
         """ Dumps this sentence. If include_spaces is set, includes whitespace tokens."""
         if not include_spaces:
             return self.words
         return self._data
 
-    def get_tabs(self):
+    def get_tabs(self) -> str:
         """ Guesses at the tabbing of this sentence. If the first element is whitespace,
         returns the whitespace after the rightmost newline in the string. """
         first = self._data[0]
@@ -248,19 +258,19 @@ class Sentence(Parsable):
     # ======== End overridden functions
 
     @property
-    def words(self):
+    def words(self) -> List[str]:
         """ Iterates over words, but without spaces. Like Unspaced List. """
         return [word.strip("\"\'") for word in self._data if not word.isspace()]
 
-    def __getitem__(self, index):
+    def __getitem__(self, index: int) -> str:
         return self.words[index]
 
-    def __contains__(self, word):
+    def __contains__(self, word: str) -> bool:
         return word in self.words
 
 
 class Block(Parsable):
-    """ Any sort of bloc, denoted by a block name and curly braces, like so:
+    """ Any sort of block, denoted by a block name and curly braces, like so:
     The parsed block:
         block name {
             content 1;
@@ -270,13 +280,13 @@ class Block(Parsable):
         names = ["block", " ", "name", " "]
         contents = [["\n    ", "content", " ", "1"], ["\n    ", "content", " ", "2"], "\n"]
     """
-    def __init__(self, parent=None):
+    def __init__(self, parent: Optional[Parsable] = None) -> None:
         super().__init__(parent)
-        self.names: Sentence = None
-        self.contents: Block = None
+        self.names: Optional[Sentence] = None
+        self.contents: Optional[Block] = None
 
     @staticmethod
-    def should_parse(lists):
+    def should_parse(lists: Any) -> bool:
         """ Returns True if `lists` can be parseable as a `Block`-- that is,
         it's got a length of 2, the first element is a `Sentence` and the second can be
         a `Statements`.
@@ -287,21 +297,21 @@ class Block(Parsable):
         return isinstance(lists, list) and len(lists) == 2 and \
                Sentence.should_parse(lists[0]) and isinstance(lists[1], list)
 
-    def set_tabs(self, tabs="    "):
+    def set_tabs(self, tabs: str = "    ") -> None:
         """ Sets tabs by setting equivalent tabbing on names, then adding tabbing
         to contents."""
         self.names.set_tabs(tabs)
         self.contents.set_tabs(tabs + "    ")
 
-    def iterate(self, expanded=False, match=None):
+    def iterate(self, expanded: bool = False,
+                match: Optional[Callable[[Parsable], bool]] = None) -> Iterator[Any]:
         """ Iterator over self, and if expanded is set, over its contents. """
         if match is None or match(self):
             yield self
         if expanded:
-            for elem in self.contents.iterate(expanded, match):
-                yield elem
+            yield from self.contents.iterate(expanded, match)
 
-    def parse(self, raw_list, add_spaces=False):
+    def parse(self, raw_list: List[Any], add_spaces: bool = False) -> None:
         """ Parses a list that resembles a block.
 
         The assumptions that this routine makes are:
@@ -313,8 +323,8 @@ class Block(Parsable):
         """
         if not Block.should_parse(raw_list):
             raise errors.MisconfigurationError("Block parsing expects a list of length 2. "
-                "First element should be a list of string types (the bloc names), "
-                "and second should be another list of statements (the bloc content).")
+                "First element should be a list of string types (the block names), "
+                "and second should be another list of statements (the block content).")
         self.names = Sentence(self)
         if add_spaces:
             raw_list[0].append(" ")
@@ -323,11 +333,12 @@ class Block(Parsable):
         self.contents.parse(raw_list[1], add_spaces)
         self._data = [self.names, self.contents]
 
-    def get_tabs(self):
+    def get_tabs(self) -> str:
         """ Guesses tabbing by retrieving tabbing guess of self.names. """
         return self.names.get_tabs()
 
-def _is_comment(parsed_obj):
+
+def _is_comment(parsed_obj: Parsable) -> bool:
     """ Checks whether parsed_obj is a comment.
 
     :param .Parsable parsed_obj:
@@ -339,7 +350,8 @@ def _is_comment(parsed_obj):
         return False
     return parsed_obj.words[0] == "#"
 
-def _is_certbot_comment(parsed_obj):
+
+def _is_certbot_comment(parsed_obj: Parsable) -> bool:
     """ Checks whether parsed_obj is a "managed by Certbot" comment.
 
     :param .Parsable parsed_obj:
@@ -356,7 +368,8 @@ def _is_certbot_comment(parsed_obj):
             return False
     return True
 
-def _certbot_comment(parent, preceding_spaces=4):
+
+def _certbot_comment(parent: Parsable, preceding_spaces: int = 4) -> Sentence:
     """ A "Managed by Certbot" comment.
     :param int preceding_spaces: Number of spaces between the end of the previous
         statement and the comment.
@@ -367,7 +380,8 @@ def _certbot_comment(parent, preceding_spaces=4):
     result.parse([" " * preceding_spaces] + COMMENT_BLOCK)
     return result
 
-def _choose_parser(parent, list_):
+
+def _choose_parser(parent: Parsable, list_: Any) -> Parsable:
     """ Choose a parser from type(parent).parsing_hooks, depending on whichever hook
     returns True first. """
     hooks = Parsable.parsing_hooks()
@@ -379,7 +393,8 @@ def _choose_parser(parent, list_):
     raise errors.MisconfigurationError(
         "None of the parsing hooks succeeded, so we don't know how to parse this set of lists.")
 
-def parse_raw(lists_, parent=None, add_spaces=False):
+
+def parse_raw(lists_: Any, parent: Optional[Parsable] = None, add_spaces: bool = False) -> Parsable:
     """ Primary parsing factory function.
 
     :param list lists_: raw lists from pyparsing to parse.
